﻿using FluentBlazorApp.DTOs;
using FluentBlazorApp.Responses;
using FluentBlazorApp.States;
using System.Net.Http;
using System.Net.Http.Headers;
using static FluentBlazorApp.Responses.CustomResponses;

namespace FluentBlazorApp.Services
{
    public class AccountService : IAccountService
    {
        private readonly HttpClient _httpClient;
        private readonly string BASE_URL = "api/account";

        public AccountService(HttpClient httpClient)
        {
            _httpClient = httpClient;
        }

        public async Task<LoginResponse> LoginAsync(LoginDTO model)
        {
            var resonse = await _httpClient.PostAsJsonAsync($"{BASE_URL}/login", model);
            var result = await resonse.Content.ReadFromJsonAsync<LoginResponse>();
            return result;
        }

        public async Task<RegistrationResonse> RegisterAsync(RegisterDTO model)
        {
            var resonse = await _httpClient.PostAsJsonAsync($"{BASE_URL}/register", model);
            var result = await resonse.Content.ReadFromJsonAsync<RegistrationResonse>();
            return result;
        }

        public async Task<WeatherForecastDTO[]> GetWeatherForecastsAync()
        {
            GetProtectedClient();
            var response = await _httpClient.GetAsync($"{BASE_URL}/weather");
            bool check = CheckIfUnauthorized(response);
            if (check)
            {
                await GetRefreshToken();
                return await GetWeatherForecastsAync();
            }

            var tmp = await response.Content.ReadFromJsonAsync<WeatherForecastDTO[]>();

            return tmp;
        }

        public async Task<LoginResponse> RefreshTokenAsync(UserSession userSession)
        {
            var resonse = await _httpClient.PostAsJsonAsync($"{BASE_URL}/refresh-token", userSession);
            var result = await resonse.Content.ReadFromJsonAsync<LoginResponse>();
            return result;
        }

        private static bool CheckIfUnauthorized(HttpResponseMessage httpResponseMessage)
        {
            if (httpResponseMessage.StatusCode == System.Net.HttpStatusCode.Unauthorized)
                return true;

            return false;
        }

        private void GetProtectedClient()
        {
            if (string.IsNullOrWhiteSpace(Constants.JWTToken))
                return;

            _httpClient.DefaultRequestHeaders.Authorization = new AuthenticationHeaderValue("Bearer", Constants.JWTToken);
        }

        private async Task GetRefreshToken()
        {
            var response = await _httpClient.PostAsJsonAsync($"{BASE_URL}/refresh-token", new UserSession() { JWTToken = Constants.JWTToken });
            var result = await response.Content.ReadFromJsonAsync<LoginResponse>();
            Constants.JWTToken = result!.JWTToken;
        }
    }
}
